@@ -48,8 +48,6 @@ project.delete()
 # end delete
 
 # fork
-fork = gl.project_forks.create({}, project_id=1)
-# or
 fork = project.forks.create({})
 
 # fork to a specific namespace
@@ -78,28 +76,18 @@ project.events.list()
 # end events list
 
 # members list
-members = gl.project_members.list()
-# or
 members = project.members.list()
 # end members list
 
 # members search
-members = gl.project_members.list(query='foo')
-# or
 members = project.members.list(query='bar')
 # end members search
 
 # members get
-member = gl.project_members.get(1)
-# or
 member = project.members.get(1)
 # end members get
 
 # members add
-member = gl.project_members.create({'user_id': user.id, 'access_level':
-                                    gitlab.DEVELOPER_ACCESS},
-                                   project_id=1)
-# or
 member = project.members.create({'user_id': user.id, 'access_level':
                                  gitlab.DEVELOPER_ACCESS})
 # end members add
@@ -110,8 +98,6 @@ member.save()
 # end members update
 
 # members delete
-gl.project_members.delete(user.id, project_id=1)
-# or
 project.members.delete(user.id)
 # or
 member.delete()
@@ -122,14 +108,10 @@ project.share(group.id, gitlab.DEVELOPER_ACCESS)
 # end share
 
 # hook list
-hooks = gl.project_hooks.list(project_id=1)
-# or
 hooks = project.hooks.list()
 # end hook list
 
 # hook get
-hook = gl.project_hooks.get(1, project_id=1)
-# or
 hook = project.hooks.get(1)
 # end hook get
 
@@ -147,8 +129,6 @@ hook.save()
 # end hook update
 
 # hook delete
-gl.project_hooks.delete(1, project_id=1)
-# or
 project.hooks.delete(1)
 # or
 hook.delete()
@@ -178,11 +158,11 @@ file_content = p.repository_raw_blob(id)
 result = project.repository_compare('master', 'branch1')
 
 # get the commits
-for i in commit:
-    print(result.commits)
+for commit in result['commits']:
+    print(commit)
 
 # get the diffs
-for file_diff in commit.diffs:
+for file_diff in result['diffs']:
     print(file_diff)
 # end repository compare
 
@@ -199,9 +179,6 @@ contributors = project.repository_contributors()
 # end repository contributors
 
 # files get
-f = gl.project_files.get(file_path='README.rst', ref='master',
-                         project_id=1)
-# or
 f = project.files.get(file_path='README.rst', ref='master')
 
 # get the base64 encoded content
@@ -212,12 +189,13 @@ print(f.decode())
 # end files get
 
 # files create
-f = gl.project_files.create({'file_path': 'testfile',
-                             'branch_name': 'master',
-                             'content': file_content,
-                             'commit_message': 'Create testfile'},
-                            project_id=1)
-# or
+# v4
+f = project.files.create({'file_path': 'testfile',
+                          'branch': 'master',
+                          'content': file_content,
+                          'commit_message': 'Create testfile'})
+
+# v3
 f = project.files.create({'file_path': 'testfile',
                           'branch_name': 'master',
                           'content': file_content,
@@ -226,50 +204,33 @@ f = project.files.create({'file_path': 'testfile',
 
 # files update
 f.content = 'new content'
-f.save(branch_name='master', commit_message='Update testfile')
+f.save(branch'master', commit_message='Update testfile')  # v4
+f.save(branch_name='master', commit_message='Update testfile')  # v3
 
 # or for binary data
 # Note: decode() is required with python 3 for data serialization. You can omit
 # it with python 2
 f.content = base64.b64encode(open('image.png').read()).decode()
-f.save(branch_name='master', commit_message='Update testfile', encoding='base64')
+f.save(branch='master', commit_message='Update testfile', encoding='base64')
 # end files update
 
 # files delete
-gl.project_files.delete({'file_path': 'testfile',
-                         'branch_name': 'master',
-                         'commit_message': 'Delete testfile'},
-                        project_id=1)
-# or
-project.files.delete({'file_path': 'testfile',
-                      'branch_name': 'master',
-                      'commit_message': 'Delete testfile'})
-# or
 f.delete(commit_message='Delete testfile')
 # end files delete
 
 # tags list
-tags = gl.project_tags.list(project_id=1)
-# or
 tags = project.tags.list()
 # end tags list
 
 # tags get
-tag = gl.project_tags.list('1.0', project_id=1)
-# or
 tags = project.tags.list('1.0')
 # end tags get
 
 # tags create
-tag = gl.project_tags.create({'tag_name': '1.0', 'ref': 'master'},
-                             project_id=1)
-# or
 tag = project.tags.create({'tag_name': '1.0', 'ref': 'master'})
 # end tags create
 
 # tags delete
-gl.project_tags.delete('1.0', project_id=1)
-# or
 project.tags.delete('1.0')
 # or
 tag.delete()
@@ -280,25 +241,14 @@ tag.set_release_description('awesome v1.0 release')
 # end tags release
 
 # snippets list
-snippets = gl.project_snippets.list(project_id=1)
-# or
 snippets = project.snippets.list()
 # end snippets list
 
 # snippets get
-snippet = gl.project_snippets.list(snippet_id, project_id=1)
-# or
 snippets = project.snippets.list(snippet_id)
 # end snippets get
 
 # snippets create
-snippet = gl.project_snippets.create({'title': 'sample 1',
-                                      'file_name': 'foo.py',
-                                      'code': 'import gitlab',
-                                      'visibility_level':
-                                      gitlab.VISIBILITY_PRIVATE},
-                                     project_id=1)
-# or
 snippet = project.snippets.create({'title': 'sample 1',
                                    'file_name': 'foo.py',
                                    'code': 'import gitlab',
@@ -316,43 +266,24 @@ snippet.save
 # end snippets update
 
 # snippets delete
-gl.project_snippets.delete(snippet_id, project_id=1)
-# or
 project.snippets.delete(snippet_id)
 # or
 snippet.delete()
 # end snippets delete
 
 # notes list
-i_notes = gl.project_issue_notes.list(project_id=1, issue_id=2)
-mr_notes = gl.project_mergerequest_notes.list(project_id=1, merge_request_id=2)
-s_notes = gl.project_snippet_notes.list(project_id=1, snippet_id=2)
-# or
 i_notes = issue.notes.list()
 mr_notes = mr.notes.list()
 s_notes = snippet.notes.list()
 # end notes list
 
 # notes get
-i_notes = gl.project_issue_notes.get(note_id, project_id=1, issue_id=2)
-mr_notes = gl.project_mergerequest_notes.get(note_id, project_id=1,
-                                             merge_request_id=2)
-s_notes = gl.project_snippet_notes.get(note_id, project_id=1, snippet_id=2)
-# or
 i_note = issue.notes.get(note_id)
 mr_note = mr.notes.get(note_id)
 s_note = snippet.notes.get(note_id)
 # end notes get
 
 # notes create
-i_note = gl.project_issue_notes.create({'body': 'note content'},
-                                       project_id=1, issue_id=2)
-mr_note = gl.project_mergerequest_notes.create({'body': 'note content'}
-                                               project_id=1,
-                                               merge_request_id=2)
-s_note = gl.project_snippet_notes.create({'body': 'note content'},
-                                          project_id=1, snippet_id=2)
-# or
 i_note = issue.notes.create({'body': 'note content'})
 mr_note = mr.notes.create({'body': 'note content'})
 s_note = snippet.notes.create({'body': 'note content'})
@@ -368,8 +299,6 @@ note.delete()
 # end notes delete
 
 # service get
-service = gl.project_services.get(service_name='asana', project_id=1)
-# or
 service = project.services.get(service_name='asana', project_id=1)
 # display it's status (enabled/disabled)
 print(service.active)
@@ -389,20 +318,14 @@ service.delete()
 # end service delete
 
 # pipeline list
-pipelines = gl.project_pipelines.list(project_id=1)
-# or
 pipelines = project.pipelines.list()
 # end pipeline list
 
 # pipeline get
-pipeline = gl.project_pipelines.get(pipeline_id, project_id=1)
-# or
 pipeline = project.pipelines.get(pipeline_id)
 # end pipeline get
 
 # pipeline create
-pipeline = gl.project_pipelines.create({'project_id': 1, 'ref': 'master'})
-# or
 pipeline = project.pipelines.create({'ref': 'master'})
 # end pipeline create
 
@@ -415,14 +338,10 @@ pipeline.cancel()
 # end pipeline cancel
 
 # boards list
-boards = gl.project_boards.list(project_id=1)
-# or
 boards = project.boards.list()
 # end boards list
 
 # boards get
-board = gl.project_boards.get(board_id, project_id=1)
-# or
 board = project.boards.get(board_id)
 # end boards get
 
