@@ -15,6 +15,8 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import functools
+
 
 class GitlabError(Exception):
     def __init__(self, error_message="", response_code=None,
@@ -39,6 +41,10 @@ class GitlabAuthenticationError(GitlabError):
     pass
 
 
+class GitlabParsingError(GitlabError):
+    pass
+
+
 class GitlabConnectionError(GitlabError):
     pass
 
@@ -47,6 +53,10 @@ class GitlabOperationError(GitlabError):
     pass
 
 
+class GitlabHttpError(GitlabError):
+    pass
+
+
 class GitlabListError(GitlabOperationError):
     pass
 
@@ -202,3 +212,24 @@ def raise_error_from_response(response, error, expected_code=200):
     raise error(error_message=message,
                 response_code=response.status_code,
                 response_body=response.content)
+
+
+def on_http_error(error):
+    """Manage GitlabHttpError exceptions.
+
+    This decorator function can be used to catch GitlabHttpError exceptions
+    raise specialized exceptions instead.
+
+    Args:
+        error(Exception): The exception type to raise -- must inherit from
+            GitlabError
+    """
+    def wrap(f):
+        @functools.wraps(f)
+        def wrapped_f(*args, **kwargs):
+            try:
+                return f(*args, **kwargs)
+            except GitlabHttpError as e:
+                raise error(e.response_code, e.error_message)
+        return wrapped_f
+    return wrap
