@@ -16,7 +16,6 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from __future__ import print_function
-from __future__ import division
 from __future__ import absolute_import
 import base64
 import json
@@ -424,7 +423,7 @@ class GroupAccessRequest(GitlabObject):
     def approve(self, access_level=gitlab.DEVELOPER_ACCESS, **kwargs):
         """Approve an access request.
 
-        Attrs:
+        Args:
             access_level (int): The access level for the user.
 
         Raises:
@@ -1286,7 +1285,7 @@ class ProjectMergeRequest(GitlabObject):
 
     def merge(self, merge_commit_message=None,
               should_remove_source_branch=False,
-              merged_when_build_succeeds=False,
+              merge_when_build_succeeds=False,
               **kwargs):
         """Accept the merge request.
 
@@ -1294,8 +1293,8 @@ class ProjectMergeRequest(GitlabObject):
             merge_commit_message (bool): Commit message
             should_remove_source_branch (bool): If True, removes the source
                                                 branch
-            merged_when_build_succeeds (bool): Wait for the build to succeed,
-                                               then merge
+            merge_when_build_succeeds (bool): Wait for the build to succeed,
+                                              then merge
 
         Returns:
             ProjectMergeRequest: The updated MR
@@ -1312,8 +1311,8 @@ class ProjectMergeRequest(GitlabObject):
             data['merge_commit_message'] = merge_commit_message
         if should_remove_source_branch:
             data['should_remove_source_branch'] = True
-        if merged_when_build_succeeds:
-            data['merged_when_build_succeeds'] = True
+        if merge_when_build_succeeds:
+            data['merge_when_build_succeeds'] = True
 
         r = self.gitlab._raw_put(url, data=data, **kwargs)
         errors = {401: GitlabMRForbiddenError,
@@ -1721,7 +1720,7 @@ class ProjectAccessRequest(GitlabObject):
     def approve(self, access_level=gitlab.DEVELOPER_ACCESS, **kwargs):
         """Approve an access request.
 
-        Attrs:
+        Args:
             access_level (int): The access level for the user.
 
         Raises:
@@ -2279,7 +2278,7 @@ class Group(GitlabObject):
     def transfer_project(self, id, **kwargs):
         """Transfers a project to this new groups.
 
-        Attrs:
+        Args:
             id (int): ID of the project to transfer.
 
         Raises:
